# FIXME

Search and replace all mentions of FIXME with sensible content in this file and in [docker-compose.yml](docker-compose.yml).

# Grafana Docker Compose files

Docker Compose files to spin up an instance of Grafana FIXME capitalization FIXME.

# How to run

Add a `COMPOSE_ENV` file and save its location as a shell variable along with the location where this repo lives, here for example `/opt/containers/grafana` plus all other variables. At [env/fqdn_context.env.example](env/fqdn_context.env.example) you'll find an example environment file.

When everything's ready start Grafana with Docker Compose, otherwise head down to [Initial setup](#initial-setup) first.

## Environment
```
export COMPOSE_DIR='/opt/containers/grafana'
export COMPOSE_CTX='ux_vilnius'
export COMPOSE_PROJECT='grafana-'"${COMPOSE_CTX}"
export COMPOSE_FILE="${COMPOSE_DIR}"'/docker-compose.yml'
export COMPOSE_ENV=<add accordingly>
```

## Context

On your deployment machine create the necessary Docker context to connect to and control the Docker daemon on whatever target host you'll be using, for example:
```
docker context create fully.qualified.domain.name --docker 'host=ssh://root@fully.qualified.domain.name'
```

## Pull

Pull images from Docker Hub verbatim.

```
docker compose --project-name "${COMPOSE_PROJECT}" --file "${COMPOSE_FILE}" --env-file "${COMPOSE_ENV}" --profile 'full' pull
```

## Copy to target

Copy images to target Docker host, that is assuming you deploy to a machine that itself has no network route to reach Docker Hub. Copying in its simplest form involves a local `docker save` and a remote `docker load`. Consider the helper mini-project [quico.space/Quico/copy-docker](https://quico.space/Quico/copy-docker) where [copy-docker.sh](https://quico.space/Quico/copy-docker/src/branch/main/copy-docker.sh) allows the following workflow:

```
source "${COMPOSE_ENV}"
for image in 'grafana:'"${GRAFANA_VERSION}" 'nginx:'"${NGINX_VERSION}"; do
    copy-docker.sh "${image}" fully.qualified.domain.name
done
```

## Start

```
docker --context 'fully.qualified.domain.name' compose --project-name "${COMPOSE_PROJECT}" --file "${COMPOSE_FILE}" --env-file "${COMPOSE_ENV}" --profile 'full' up --detach
```

# Initial setup

We're assuming you run Docker Compose workloads with ZFS-based bind mounts. ZFS management, creating a zpool and setting adequate properties for its datasets is out of scope of this document.

## Datasets

Create ZFS datasets and set permissions as needed.

* Parent dateset
    ```
    zfs create -o mountpoint=/opt/docker-data 'zpool/docker-data'
    ```

* Container-specific datasets
    ```
    zfs create -p 'zpool/docker-data/grafana-'"${COMPOSE_CTX}"'/grafana/data/db'
    zfs create -p 'zpool/docker-data/grafana-'"${COMPOSE_CTX}"'/grafana/data/logs'
    zfs create -p 'zpool/docker-data/grafana-'"${COMPOSE_CTX}"'/grafana/config'
    zfs create -p 'zpool/docker-data/grafana-'"${COMPOSE_CTX}"'/nginx/data/db'
    zfs create -p 'zpool/docker-data/grafana-'"${COMPOSE_CTX}"'/nginx/data/logs'
    zfs create -p 'zpool/docker-data/grafana-'"${COMPOSE_CTX}"'/nginx/config'
    ```
    FIXME delete when all is done FIXME When changing bind mount locations to real ones remember to also update `volumes:` in [docker-compose.yml](docker-compose.yml).

* Create subdirs
    ```
    mkdir -p '/opt/docker-data/grafana-'"${COMPOSE_CTX}"'/grafana/'{'.ssh','config','data','projects'}
    ```

* Change ownership
    ```
    chown -R 1000:1000 '/opt/docker-data/grafana-${COMPOSE_CTX}/grafana/data/'{*,.*}
    ```

## Additional files

Place the following files on target server. Use the directory structure at [build-context](build-context) as a guide, specifically at `docker-data`.

FIXME Add details about files that aren't self-explanatory FIXME

```
build-context/
├── grafana
│   ├── docker-data
│   |   └── config
│   │       └── grafana.cfg
│   ├── ...
│   └── ...
└── nginx
    ├── docker-data
    |   └── config
    │       └── nginx.cfg
    ├── ...
    └── ...
```

When done head back up to [How to run](#how-to-run).
